/*============================================================================
 * eps.h
 *      Global definitions for eps.cc.
 *==========================================================================*/

#ifndef _eps_h_
#define _eps_h_

#include "basic_types.h"
#include "basic_ops.h"
#include "sim.h"

typedef struct
{
    Cplx eps_r;
    Real err;
} eps_err_pair_def;

void eps_fit(sim_def *sim, Cplx *S);

inline __host__ __device__ void get_S_tline1_3d(sim_def *sim, Cplx sqrt_eps1, Real f, Cplx *S_out)
{
    const Real c0 = 2.998e8;
    const Cplx eta0 = 376.7;
    
    Cplx beta, theta;
    Cplx eta;
    Cplx G0, G02;
    
    Cplx S21_sum, S21_prev;
    Cplx T0, T02;

    Real gn;
    
    Uint n;
    
    beta = 2.0*M_PI*f*sqrt_eps1/c0;
    theta = beta*sim->d;
    eta = eta0/sqrt_eps1;

    G0 = (eta-eta0)/(eta+eta0);
    T0 = EXPC(-J*theta);

    G02 = G0*G0;
    T02 = T0*T0;
    
    gn = (sim->D)/(sim->D + sim->d);
    
    S21_prev = (1.0 + G0)*T0;
    S21_sum = S21_prev*gn;
    
    for (n=1; n<=sim->ord; n++)
    {
        gn = (sim->D)/(sim->D + (Real)(2*n+1)*sim->d);
        S21_prev = S21_prev*G02*T02;
        S21_sum += S21_prev*gn;
    }
    
    *S_out = S21_sum*(1.0 - G0);
}


inline __host__ __device__ void get_S_tline1_2d(sim_def *sim, Cplx sqrt_eps1, Real f, Cplx *S_out)
{
    const Real c0 = 2.998e8;
    const Cplx eta0 = 376.7;
    
    Cplx beta, theta;
    Cplx eta;
    Cplx G0, G02;
    
    Cplx S21_sum, S21_prev;
    Cplx T0, T02;

    Real gn;
    
    Uint n;
    
    beta = 2.0*M_PI*f*sqrt_eps1/c0;
    theta = beta*sim->d;
    eta = eta0/sqrt_eps1;

    G0 = (eta-eta0)/(eta+eta0);
    T0 = EXPC(-J*theta);

    G02 = G0*G0;
    T02 = T0*T0;
    
    gn = SQRT((sim->D)/(sim->D + sim->d));
    
    S21_prev = (1.0 + G0)*T0;
    S21_sum = S21_prev*gn;
    
    for (n=1; n<=sim->ord; n++)
    {
        gn = SQRT((sim->D)/(sim->D + (Real)(2*n+1)*sim->d));
        S21_prev = S21_prev*G02*T02;
        S21_sum += S21_prev*gn;
    }
    
    *S_out = S21_sum*(1.0 - G0);
}

inline __host__ __device__ void get_S_tline1_1d(sim_def *sim, Cplx sqrt_eps1, Real f, Cplx *S_out)
{
    const Real c0 = 2.998e8;
    const Cplx eta0 = 376.7;
    
    Cplx beta, theta;
    Cplx eta;
    Cplx G0, G02;
    
    Cplx S21_sum, S21_prev;
    Cplx T0, T02;

    Uint n;
    
    beta = 2.0*M_PI*f*sqrt_eps1/c0;
    theta = beta*sim->d;
    eta = eta0/sqrt_eps1;

    G0 = (eta-eta0)/(eta+eta0);
    T0 = EXPC(-J*theta);

    G02 = G0*G0;
    T02 = T0*T0;
    
    S21_prev = (1.0 + G0)*T0;
    S21_sum = S21_prev;
    
    for (n=1; n<=sim->ord; n++)
    {
        S21_prev = S21_prev*G02*T02;
        S21_sum += S21_prev;
    }
    
    *S_out = S21_sum*(1.0 - G0);
}

#endif
