/*============================================================================
 * sim.h
 *      Code for reading and storing high-level simulation parameters.
 *==========================================================================*/

#ifndef _sim_h_
#define _sim_h_

#include "basic_types.h"
#include "log.h"

#define MAX_PARM_RANGE          100000
#define MAX_FREQ                100000
#define MAX_BAND                10000

typedef struct
{
    Real start, stop;
    Uint N;
    Uint is_log;
    
    Real vals[MAX_PARM_RANGE];
    
} parm_sweep_def;

// Constant parameters used for simulation
typedef struct
{
    log_level_enum log_level;

    // GPU acceleration?
    Bool use_gpu;
    // Total distance from antennas to sample.
    Real D;

    // Thickness of sample
    Real d;

    // Ranges of permittivity parameter
    parm_sweep_def eps_real;
    parm_sweep_def eps_imag;

    // Band fitting points
    parm_sweep_def band;

    // Frequency range
    Real f0, f1;
    Uint Nf;

    // Number of iterations on optimization
    Uint N_iter;
    
    // Order to use in S21 series
    Uint ord;
    // Type of spreading (0=none, 1=3D, 2=2D)
    Uint spread_type;
    
    // Temporary data below this line
    Real df;

    // For each frequency, define L and R endpoints of the band sample points
    // it belongs in.  Also, store the interpolation weights to be used.
    Int band_idx_l[MAX_FREQ];
    Int band_idx_r[MAX_FREQ];
    Real band_weight_l[MAX_FREQ];
    Real band_weight_r[MAX_FREQ];

    // For each band, define the start and stop frequency indicies in that band
    Uint band_fi1[MAX_BAND];
    Uint band_fi2[MAX_BAND];

} sim_def;

Bool sim_read(const char *fname, sim_def *sim);
Bool sim_run(const char *parm_file, const char *s_parm_file);

#endif
